package be.iminds.ilabt.jfed.restapi.debugdata;

import be.iminds.ilabt.jfed.restapi.json.Experiment;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class ExperimentDDBuilder {
    private static final Logger LOG = LoggerFactory.getLogger(ExperimentDDBuilder.class);

    private Long id;
    private GeniUrn sliceUrn;
    private GeniUrn userUrn;
    private List<ServerResourceMethodDDBuilder> serverResourceMethodDDs;

    public ExperimentDDBuilder() {
    }

    public ExperimentDDBuilder(ExperimentDD orig) {
        this.id = orig.getId();
        this.sliceUrn = orig.getSliceUrn();
        this.userUrn = orig.getUserUrn();
        setServerResourceMethodDDs(orig.getServerResourceMethodDDs());
    }

    public ExperimentDDBuilder(Experiment experiment) {
        assert experiment != null;
        assert experiment.getId() != null;
        setFromExperiment(experiment);
        assert getId() != null;
        setServerResourceMethodDDs(new ArrayList<>());
    }

    public ExperimentDDBuilder setFromExperiment(Experiment experiment) {
        this.id = experiment.getId();
        this.sliceUrn = experiment.getSliceUrn();
        this.userUrn = experiment.getOwner(); // or experiment.getSpeakingFor(); ?
        return this;
    }

    public ExperimentDDBuilder setId(Long id) {
        this.id = id;
        return this;
    }

    public ExperimentDDBuilder setSliceUrn(GeniUrn sliceUrn) {
        this.sliceUrn = sliceUrn;
        return this;
    }

    public ExperimentDDBuilder setUserUrn(GeniUrn userUrn) {
        this.userUrn = userUrn;
        return this;
    }

    public ExperimentDDBuilder addServerResourceMethodDD(ServerResourceMethodDDBuilder builder) {
        if (this.serverResourceMethodDDs == null) {
            this.serverResourceMethodDDs = new ArrayList<>();
        }
        this.serverResourceMethodDDs.add(builder);
        return this;
    }

    public ExperimentDDBuilder setServerResourceMethodDDs(List<ServerResourceMethodDD> serverResourceMethodDDs) {
        if (serverResourceMethodDDs == null) {
            this.serverResourceMethodDDs = null;
        } else {
            this.serverResourceMethodDDs = new ArrayList<>();
            for (ServerResourceMethodDD serverResourceMethodDD : serverResourceMethodDDs) {
                this.serverResourceMethodDDs.add(new ServerResourceMethodDDBuilder(serverResourceMethodDD));
            }
        }
        return this;
    }

    public ExperimentDD create() {
        return new ExperimentDD(id, sliceUrn, userUrn,
                serverResourceMethodDDs == null ? null : ServerResourceMethodDDBuilder.createAll(serverResourceMethodDDs));
    }


    public Long getId() { return id; }
    public GeniUrn getSliceUrn() { return sliceUrn; }
    public GeniUrn getUserUrn() { return userUrn; }
    public List<ServerResourceMethodDDBuilder> getServerResourceMethodDDs() { return serverResourceMethodDDs; }

    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(create());
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+ getClass().getSimpleName() +" to JSON: "+e.getMessage();
        }
    }
}
