package be.iminds.ilabt.jfed.restapi.dropwizard;

import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.classic.sift.MDCBasedDiscriminator;
import ch.qos.logback.classic.sift.SiftingAppender;
import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.Appender;
import ch.qos.logback.core.Context;
import ch.qos.logback.core.Layout;
import ch.qos.logback.core.sift.AppenderFactory;
import com.fasterxml.jackson.annotation.JsonTypeName;
import io.dropwizard.logging.AbstractAppenderFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * jFedRestApiAppenderFactory provides access to slf4j logging messages, based on the MDC.
 * This is used to store per experiment and/or per call logging.
 *
 * This needs to be registered in the file "META-INFI/services/io.dropwizard.logging.AppenderFactory"
 */
@JsonTypeName("jFedRestApiSift")
public class JFedRestApiSiftingAppenderFactory extends AbstractAppenderFactory {
    /**
     * MDC key used by this jFedRestApiAppender.
     *
     * This is also the name that will be used by jFed code to access this appender.
     */
    private String mdcKey;

    /**
     * default value to be used when MDC key is not present
     */
    private String defaultValue;

    private final Map<String, JFedLoggingAppender> appenders = new HashMap<>();

    public final String getMdcKey() {
        return mdcKey;
    }

    public final void setMdcKey(final String mdcKey) {
        this.mdcKey = mdcKey;
    }

    public final String getDefaultValue() {
        return defaultValue;
    }

    public final void setDefaultValue(final String defaultValue) {
        this.defaultValue = defaultValue;
    }

    public JFedLoggingAppender getAppender(String value) {
        return appenders.get(value);
    }

    @Override
    public Appender<ILoggingEvent> build(LoggerContext loggerContext, String appName, Layout<ILoggingEvent> layout) {
        final SiftingAppender siftingAppender = new SiftingAppender();
        siftingAppender.setName("jfedrestapi-sift-appender");
        siftingAppender.setContext(loggerContext);
        addThresholdFilter(siftingAppender, threshold);

        MDCBasedDiscriminator mdcBasedDiscriminator = new MDCBasedDiscriminator();
        mdcBasedDiscriminator.setKey(mdcKey);
        mdcBasedDiscriminator.setDefaultValue(defaultValue);
        mdcBasedDiscriminator.start();
        siftingAppender.setDiscriminator(mdcBasedDiscriminator);

        siftingAppender.setAppenderFactory(new AppenderFactory<ILoggingEvent>() {
            @Override
            public Appender<ILoggingEvent> buildAppender(final Context context, final String discriminatingValue)  {
                JFedLoggingSiftedAppender appender = new JFedLoggingSiftedAppender(mdcKey, discriminatingValue);

                appender.setName("jfed-" + discriminatingValue);
                appender.setContext(context);

                appender.start();

                appenders.put(discriminatingValue, appender);

                return appender;
            }

        });
        siftingAppender.start();

        return wrapAsync(siftingAppender);
    }
}
